# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .balances import (
    BalancesResource,
    AsyncBalancesResource,
    BalancesResourceWithRawResponse,
    AsyncBalancesResourceWithRawResponse,
    BalancesResourceWithStreamingResponse,
    AsyncBalancesResourceWithStreamingResponse,
)
from .transactions import (
    TransactionsResource,
    AsyncTransactionsResource,
    TransactionsResourceWithRawResponse,
    AsyncTransactionsResourceWithRawResponse,
    TransactionsResourceWithStreamingResponse,
    AsyncTransactionsResourceWithStreamingResponse,
)
from .charge_schedules import (
    ChargeSchedulesResource,
    AsyncChargeSchedulesResource,
    ChargeSchedulesResourceWithRawResponse,
    AsyncChargeSchedulesResourceWithRawResponse,
    ChargeSchedulesResourceWithStreamingResponse,
    AsyncChargeSchedulesResourceWithStreamingResponse,
)
from .transaction_schedules import (
    TransactionSchedulesResource,
    AsyncTransactionSchedulesResource,
    TransactionSchedulesResourceWithRawResponse,
    AsyncTransactionSchedulesResourceWithRawResponse,
    TransactionSchedulesResourceWithStreamingResponse,
    AsyncTransactionSchedulesResourceWithStreamingResponse,
)

__all__ = [
    "TransactionsResource",
    "AsyncTransactionsResource",
    "TransactionsResourceWithRawResponse",
    "AsyncTransactionsResourceWithRawResponse",
    "TransactionsResourceWithStreamingResponse",
    "AsyncTransactionsResourceWithStreamingResponse",
    "ChargeSchedulesResource",
    "AsyncChargeSchedulesResource",
    "ChargeSchedulesResourceWithRawResponse",
    "AsyncChargeSchedulesResourceWithRawResponse",
    "ChargeSchedulesResourceWithStreamingResponse",
    "AsyncChargeSchedulesResourceWithStreamingResponse",
    "TransactionSchedulesResource",
    "AsyncTransactionSchedulesResource",
    "TransactionSchedulesResourceWithRawResponse",
    "AsyncTransactionSchedulesResourceWithRawResponse",
    "TransactionSchedulesResourceWithStreamingResponse",
    "AsyncTransactionSchedulesResourceWithStreamingResponse",
    "BalancesResource",
    "AsyncBalancesResource",
    "BalancesResourceWithRawResponse",
    "AsyncBalancesResourceWithRawResponse",
    "BalancesResourceWithStreamingResponse",
    "AsyncBalancesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["AccountPlanListParams"]


class AccountPlanListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account: str
    """
    The unique identifier (UUID) for the Account whose AccountPlans and
    AccountPlanGroups you want to retrieve.

    **NOTE:** Only returns the currently active AccountPlans and AccountPlanGroups
    for the specified Account. Use in combination with the `includeall` query
    parameter to return both active and inactive.
    """

    contract: Optional[str]
    """
    The unique identifier (UUID) of the Contract which the AccountPlans you want to
    retrieve have been linked to.

    **NOTE:** Does not return AccountPlanGroups that have been linked to the
    Contract.
    """

    date: str
    """
    The specific date for which you want to retrieve AccountPlans and
    AccountPlanGroups.

    **NOTE:** Returns both active and inactive AccountPlans and AccountPlanGroups
    for the specified date.
    """

    ids: SequenceNotStr[str]
    """
    A list of unique identifiers (UUIDs) for specific AccountPlans and
    AccountPlanGroups you want to retrieve.
    """

    includeall: bool
    """
    A Boolean flag that specifies whether to include both active and inactive
    AccountPlans and AccountPlanGroups in the list.

    - **TRUE** - both active and inactive AccountPlans and AccountPlanGroups are
      included in the list.
    - **FALSE** - only active AccountPlans and AccountPlanGroups are retrieved in
      the list._(Default)_

    **NOTE:** Only operative if you also have one of `account`, `plan` or `contract`
    as a query parameter.
    """

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """
    The `nextToken` for retrieving the next page of AccountPlans and
    AccountPlanGroups. It is used to fetch the next page of AccountPlans and
    AccountPlanGroups in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """The maximum number of AccountPlans and AccountPlanGroups to return per page."""

    plan: str
    """
    The unique identifier (UUID) for the Plan whose associated AccountPlans you want
    to retrieve.

    **NOTE:** Does not return AccountPlanGroups if you use a `planGroupId`.
    """

    product: str
    """
    The unique identifier (UUID) for the Product whose associated AccountPlans you
    want to retrieve.

    **NOTE:** You cannot use the `product` query parameter as a single filter
    condition, but must always use it in combination with the `account` query
    parameter.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["StatementJobResponse"]


class StatementJobResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    bill_id: Optional[str] = FieldInfo(alias="billId", default=None)
    """The unique identifier (UUID) of the bill associated with the StatementJob."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) of the user who created this StatementJob."""

    csv_statement_status: Optional[Literal["LATEST", "STALE", "INVALIDATED"]] = FieldInfo(
        alias="csvStatementStatus", default=None
    )

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO-8601 format)_ when the StatementJob was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The date and time _(in ISO-8601 format)_ when the StatementJob was last
    modified.
    """

    include_csv_format: Optional[bool] = FieldInfo(alias="includeCsvFormat", default=None)
    """
    A Boolean value indicating whether the generated statement includes a CSV
    format.

    - TRUE - includes the statement in CSV format.
    - FALSE - no CSV format statement.
    """

    json_statement_status: Optional[Literal["LATEST", "STALE", "INVALIDATED"]] = FieldInfo(
        alias="jsonStatementStatus", default=None
    )

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) of the user who last modified this StatementJob."""

    org_id: Optional[str] = FieldInfo(alias="orgId", default=None)
    """The unique identifier (UUID) of your Organization.

    The Organization represents your company as a direct customer of our service.
    """

    presigned_csv_statement_url: Optional[str] = FieldInfo(alias="presignedCsvStatementUrl", default=None)

    presigned_json_statement_url: Optional[str] = FieldInfo(alias="presignedJsonStatementUrl", default=None)
    """The URL to access the generated statement in JSON format.

    This URL is temporary and has a limited lifetime.
    """

    statement_job_status: Optional[Literal["PENDING", "RUNNING", "COMPLETE", "CANCELLED", "FAILED"]] = FieldInfo(
        alias="statementJobStatus", default=None
    )
    """The current status of the StatementJob.

    The status helps track the progress and outcome of a StatementJob.
    """

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["LookupTableRevisionDataJobDownloadResponse"]


class LookupTableRevisionDataJobDownloadResponse(BaseModel):
    """Response containing the download job details"""

    job_id: Optional[str] = FieldInfo(alias="jobId", default=None)
    """UUID of the download job"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["TransactionScheduleUpdateParams"]


class TransactionScheduleUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    balance_id: Required[Annotated[str, PropertyInfo(alias="balanceId")]]

    amount: Required[float]
    """The amount of each Balance Transaction created by this Schedule."""

    code: Required[str]
    """The unique short code of the Balance Transaction Schedule."""

    end_date: Required[Annotated[Union[str, datetime], PropertyInfo(alias="endDate", format="iso8601")]]
    """The end date (_in ISO-8601 format_) of the Balance Transaction Schedule.

    **NOTE:** End date is exclusive.
    """

    frequency: Required[Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"]]
    """Represents standard scheduling frequencies options for a job."""

    frequency_interval: Required[Annotated[int, PropertyInfo(alias="frequencyInterval")]]
    """
    Used in conjunction with `frequency` to define how often Balance Transactions
    are generated by the Schedule. For example, if `frequency` is MONTHLY and
    `frequencyInterval` is 3, then Balance Transactions are generated every three
    months.
    """

    name: Required[str]
    """The name of the Balance Transaction Schedule."""

    start_date: Required[Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]]
    """The start date (_in ISO-8601 format_) of the Balance Transaction Schedule."""

    transaction_description: Required[Annotated[str, PropertyInfo(alias="transactionDescription")]]
    """
    The description of each Balance Transaction that will be created by this
    Schedule.
    """

    transaction_type_id: Required[Annotated[str, PropertyInfo(alias="transactionTypeId")]]
    """
    The unique identifier (UUID) of the transaction type used to create Transactions
    by this Schedule. You can obtain a list of Transaction Types created for the
    Organization. See the the
    [List TransactionTypes](https://www.m3ter.com/docs/api#tag/TransactionType/operation/ListTransactionTypes)
    endpoint of this API Reference.
    """

    currency_paid: Annotated[str, PropertyInfo(alias="currencyPaid")]
    """The currency code of the payment if it differs from the Balance currency.

    For example: USD, GBP or EUR.
    """

    custom_fields: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="customFields")]
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level, `customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    paid: float
    """The payment amount if the payment currency differs from the Balance currency."""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Balance"]


class Balance(BaseModel):
    id: str
    """The UUID of the entity."""

    account_id: Optional[str] = FieldInfo(alias="accountId", default=None)
    """
    The unique identifier (UUID) for the end customer Account the Balance belongs
    to.
    """

    allow_overdraft: Optional[bool] = FieldInfo(alias="allowOverdraft", default=None)
    """Allow balance amounts to fall below zero.

    This feature is enabled on request. Please get in touch with m3ter Support or
    your m3ter contact if you would like it enabling for your organization(s).
    """

    amount: Optional[float] = None
    """The financial value that the Balance holds."""

    balance_draw_down_description: Optional[str] = FieldInfo(alias="balanceDrawDownDescription", default=None)
    """
    A description for the bill line items for charges drawn-down against the
    Balance.
    """

    code: Optional[str] = None
    """A unique short code assigned to the Balance."""

    consumptions_accounting_product_id: Optional[str] = FieldInfo(alias="consumptionsAccountingProductId", default=None)
    """
    Product ID that any Balance Consumed line items will be attributed to for
    accounting purposes.(_Optional_)
    """

    contract_id: Optional[str] = FieldInfo(alias="contractId", default=None)
    """
    The unique identifier (UUID) for a Contract on the Account the Balance has been
    added to.
    """

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) for the user who created the Balance."""

    currency: Optional[str] = None
    """The currency code used for the Balance amount. For example: USD, GBP or EUR."""

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    description: Optional[str] = None
    """A description of the Balance."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO 8601 format)_ when the Balance was first created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The date and time _(in ISO 8601 format)_ when the Balance was last modified."""

    end_date: Optional[datetime] = FieldInfo(alias="endDate", default=None)
    """
    The date _(in ISO 8601 format)_ after which the Balance will no longer be
    active.
    """

    fees_accounting_product_id: Optional[str] = FieldInfo(alias="feesAccountingProductId", default=None)
    """
    Product ID that any Balance Fees line items will be attributed to for accounting
    purposes.(_Optional_)
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) for the user who last modified the Balance."""

    line_item_types: Optional[
        List[
            Literal[
                "STANDING_CHARGE",
                "USAGE",
                "MINIMUM_SPEND",
                "COUNTER_RUNNING_TOTAL_CHARGE",
                "COUNTER_ADJUSTMENT_DEBIT",
                "AD_HOC",
            ]
        ]
    ] = FieldInfo(alias="lineItemTypes", default=None)
    """
    A list of line item charge types that can draw-down against the Balance amount
    at billing.
    """

    name: Optional[str] = None
    """The official name of the Balance."""

    overage_description: Optional[str] = FieldInfo(alias="overageDescription", default=None)
    """A description for overage charges."""

    overage_surcharge_percent: Optional[float] = FieldInfo(alias="overageSurchargePercent", default=None)
    """
    The percentage surcharge applied to overage charges _(usage above the Balance)_.
    """

    product_ids: Optional[List[str]] = FieldInfo(alias="productIds", default=None)
    """
    A list of Product IDs whose consumption charges due at billing can be drawn-down
    against the Balance amount.
    """

    rollover_amount: Optional[float] = FieldInfo(alias="rolloverAmount", default=None)
    """
    The maximum amount that can be carried over past the Balance end date and
    draw-down against for billing if there is an unused Balance amount remaining
    when the Balance end date is reached.
    """

    rollover_end_date: Optional[datetime] = FieldInfo(alias="rolloverEndDate", default=None)
    """
    The end date _(in ISO 8601 format)_ for the rollover grace period, which is the
    period that unused Balance amounts can be carried over beyond the specified
    Balance `endDate` and continue to be drawn-down against for billing.
    """

    start_date: Optional[datetime] = FieldInfo(alias="startDate", default=None)
    """The date _(in ISO 8601 format)_ when the Balance becomes active."""

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["LookupTableRevisionDataRetrieveResponse"]


class LookupTableRevisionDataRetrieveResponse(BaseModel):
    """Response containing data for a Lookup Table Revision"""

    items: List[Dict[str, object]]
    """The Lookup Table Revision Data."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created the Lookup Table Revision Data."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when the Lookup Table Revision Data was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when the Lookup Table Revision Data was last modified."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified the Lookup Table Revision Data."""

    version: Optional[int] = None
    """The version of the Lookup Table Revision Data."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .data_explorer_group_param import DataExplorerGroupParam

__all__ = [
    "DataExportCreateAdhocParams",
    "AdHocOperationalDataRequest",
    "AdHocUsageDataRequest",
    "AdHocUsageDataRequestAggregation",
    "AdHocUsageDataRequestDimensionFilter",
]


class AdHocOperationalDataRequest(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    operational_data_types: Required[
        Annotated[
            List[
                Literal[
                    "BILLS",
                    "COMMITMENTS",
                    "ACCOUNTS",
                    "BALANCES",
                    "CONTRACTS",
                    "ACCOUNT_PLANS",
                    "AGGREGATIONS",
                    "PLANS",
                    "PRICING",
                    "PRICING_BANDS",
                    "BILL_LINE_ITEMS",
                    "METERS",
                    "PRODUCTS",
                    "COMPOUND_AGGREGATIONS",
                    "PLAN_GROUPS",
                    "PLAN_GROUP_LINKS",
                    "PLAN_TEMPLATES",
                    "BALANCE_TRANSACTIONS",
                    "TRANSACTION_TYPES",
                    "CHARGES",
                ]
            ],
            PropertyInfo(alias="operationalDataTypes"),
        ]
    ]
    """The list of the operational data types should be exported for."""

    source_type: Required[Annotated[Literal["OPERATIONAL"], PropertyInfo(alias="sourceType")]]
    """The type of data to export. Possible values are: OPERATIONAL"""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """


class AdHocUsageDataRequest(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    source_type: Required[Annotated[Literal["USAGE"], PropertyInfo(alias="sourceType")]]
    """The type of data to export. Possible values are: USAGE"""

    account_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="accountIds")]
    """List of account IDs for which the usage data will be exported."""

    aggregations: Iterable[AdHocUsageDataRequestAggregation]
    """List of aggregations to apply"""

    dimension_filters: Annotated[Iterable[AdHocUsageDataRequestDimensionFilter], PropertyInfo(alias="dimensionFilters")]
    """List of dimension filters to apply"""

    end_date: Annotated[Union[str, datetime], PropertyInfo(alias="endDate", format="iso8601")]
    """The exclusive end date for the data export."""

    groups: Iterable[DataExplorerGroupParam]
    """List of groups to apply"""

    meter_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="meterIds")]
    """List of meter IDs for which the usage data will be exported."""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """


class AdHocUsageDataRequestAggregation(TypedDict, total=False):
    field_code: Required[Annotated[str, PropertyInfo(alias="fieldCode")]]
    """Field code"""

    field_type: Required[Annotated[Literal["DIMENSION", "MEASURE"], PropertyInfo(alias="fieldType")]]
    """Type of field"""

    function: Required[Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN", "UNIQUE"]]
    """Aggregation function"""

    meter_id: Required[Annotated[str, PropertyInfo(alias="meterId")]]
    """Meter ID"""


class AdHocUsageDataRequestDimensionFilter(TypedDict, total=False):
    field_code: Required[Annotated[str, PropertyInfo(alias="fieldCode")]]
    """Field code"""

    meter_id: Required[Annotated[str, PropertyInfo(alias="meterId")]]
    """Meter ID"""

    values: Required[SequenceNotStr[str]]
    """Values to filter by"""


DataExportCreateAdhocParams: TypeAlias = Union[AdHocOperationalDataRequest, AdHocUsageDataRequest]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from tests.utils import assert_matches_type
from m3ter.types.lookup_tables import (
    LookupTableRevisionDataCopyResponse,
    LookupTableRevisionDataDeleteResponse,
    LookupTableRevisionDataUpdateResponse,
    LookupTableRevisionDataArchieveResponse,
    LookupTableRevisionDataRetrieveResponse,
    LookupTableRevisionDataDeleteKeyResponse,
    LookupTableRevisionDataUpdateKeyResponse,
    LookupTableRevisionDataRetrieveKeyResponse,
    LookupTableRevisionDataGenerateDownloadURLResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestLookupTableRevisionData:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.retrieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )
        assert_matches_type(LookupTableRevisionDataRetrieveResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.retrieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            additional=["string"],
            limit=0,
        )
        assert_matches_type(LookupTableRevisionDataRetrieveResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = response.parse()
        assert_matches_type(LookupTableRevisionDataRetrieveResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.with_streaming_response.retrieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = response.parse()
            assert_matches_type(LookupTableRevisionDataRetrieveResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
            )

    @parametrize
    def test_method_update(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.update(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            items=[{"foo": "bar"}],
        )
        assert_matches_type(LookupTableRevisionDataUpdateResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.update(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            items=[{"foo": "bar"}],
            additional=["string"],
            version=0,
        )
        assert_matches_type(LookupTableRevisionDataUpdateResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: M3ter) -> None:
        response = client.lookup_tables.lookup_table_revision_data.with_raw_response.update(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            items=[{"foo": "bar"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = response.parse()
        assert_matches_type(LookupTableRevisionDataUpdateResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.with_streaming_response.update(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            items=[{"foo": "bar"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = response.parse()
            assert_matches_type(LookupTableRevisionDataUpdateResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.update(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
                items=[{"foo": "bar"}],
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.update(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
                items=[{"foo": "bar"}],
            )

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.delete(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )
        assert_matches_type(LookupTableRevisionDataDeleteResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = client.lookup_tables.lookup_table_revision_data.with_raw_response.delete(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = response.parse()
        assert_matches_type(LookupTableRevisionDataDeleteResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.with_streaming_response.delete(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = response.parse()
            assert_matches_type(LookupTableRevisionDataDeleteResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.delete(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.delete(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
            )

    @parametrize
    def test_method_archieve(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.archieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_type="application/jsonl",
        )
        assert_matches_type(LookupTableRevisionDataArchieveResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_raw_response_archieve(self, client: M3ter) -> None:
        response = client.lookup_tables.lookup_table_revision_data.with_raw_response.archieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_type="application/jsonl",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = response.parse()
        assert_matches_type(LookupTableRevisionDataArchieveResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_streaming_response_archieve(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.with_streaming_response.archieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_type="application/jsonl",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = response.parse()
            assert_matches_type(LookupTableRevisionDataArchieveResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_archieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.archieve(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
                content_type="application/jsonl",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.archieve(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
                content_type="application/jsonl",
            )

    @parametrize
    def test_method_copy(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.copy(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )
        assert_matches_type(LookupTableRevisionDataCopyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_method_copy_with_all_params(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.copy(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            revision_id="revisionId",
        )
        assert_matches_type(LookupTableRevisionDataCopyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_raw_response_copy(self, client: M3ter) -> None:
        response = client.lookup_tables.lookup_table_revision_data.with_raw_response.copy(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = response.parse()
        assert_matches_type(LookupTableRevisionDataCopyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_streaming_response_copy(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.with_streaming_response.copy(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = response.parse()
            assert_matches_type(LookupTableRevisionDataCopyResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_copy(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.copy(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.copy(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
            )

    @parametrize
    def test_method_delete_key(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.delete_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        )
        assert_matches_type(LookupTableRevisionDataDeleteKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_method_delete_key_with_all_params(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.delete_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
            version=0,
        )
        assert_matches_type(LookupTableRevisionDataDeleteKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_raw_response_delete_key(self, client: M3ter) -> None:
        response = client.lookup_tables.lookup_table_revision_data.with_raw_response.delete_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = response.parse()
        assert_matches_type(LookupTableRevisionDataDeleteKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_streaming_response_delete_key(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.with_streaming_response.delete_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = response.parse()
            assert_matches_type(LookupTableRevisionDataDeleteKeyResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete_key(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.delete_key(
                lookup_key="lookupKey",
                lookup_table_id="",
                lookup_table_revision_id="lookupTableRevisionId",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.delete_key(
                lookup_key="lookupKey",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_key` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.delete_key(
                lookup_key="",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )

    @parametrize
    def test_method_generate_download_url(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.generate_download_url(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_length=1,
            content_type="application/jsonl",
            file_name="x",
        )
        assert_matches_type(
            LookupTableRevisionDataGenerateDownloadURLResponse, lookup_table_revision_data, path=["response"]
        )

    @parametrize
    def test_method_generate_download_url_with_all_params(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.generate_download_url(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_length=1,
            content_type="application/jsonl",
            file_name="x",
            version=0,
        )
        assert_matches_type(
            LookupTableRevisionDataGenerateDownloadURLResponse, lookup_table_revision_data, path=["response"]
        )

    @parametrize
    def test_raw_response_generate_download_url(self, client: M3ter) -> None:
        response = client.lookup_tables.lookup_table_revision_data.with_raw_response.generate_download_url(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_length=1,
            content_type="application/jsonl",
            file_name="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = response.parse()
        assert_matches_type(
            LookupTableRevisionDataGenerateDownloadURLResponse, lookup_table_revision_data, path=["response"]
        )

    @parametrize
    def test_streaming_response_generate_download_url(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.with_streaming_response.generate_download_url(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_length=1,
            content_type="application/jsonl",
            file_name="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = response.parse()
            assert_matches_type(
                LookupTableRevisionDataGenerateDownloadURLResponse, lookup_table_revision_data, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_generate_download_url(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.generate_download_url(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
                content_length=1,
                content_type="application/jsonl",
                file_name="x",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.generate_download_url(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
                content_length=1,
                content_type="application/jsonl",
                file_name="x",
            )

    @parametrize
    def test_method_retrieve_key(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.retrieve_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        )
        assert_matches_type(LookupTableRevisionDataRetrieveKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_raw_response_retrieve_key(self, client: M3ter) -> None:
        response = client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = response.parse()
        assert_matches_type(LookupTableRevisionDataRetrieveKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_streaming_response_retrieve_key(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.with_streaming_response.retrieve_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = response.parse()
            assert_matches_type(
                LookupTableRevisionDataRetrieveKeyResponse, lookup_table_revision_data, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve_key(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve_key(
                lookup_key="lookupKey",
                lookup_table_id="",
                lookup_table_revision_id="lookupTableRevisionId",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve_key(
                lookup_key="lookupKey",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_key` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve_key(
                lookup_key="",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )

    @parametrize
    def test_method_update_key(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.update_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
            item={"foo": "bar"},
        )
        assert_matches_type(LookupTableRevisionDataUpdateKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_method_update_key_with_all_params(self, client: M3ter) -> None:
        lookup_table_revision_data = client.lookup_tables.lookup_table_revision_data.update_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
            item={"foo": "bar"},
            additional=["string"],
            version=0,
        )
        assert_matches_type(LookupTableRevisionDataUpdateKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_raw_response_update_key(self, client: M3ter) -> None:
        response = client.lookup_tables.lookup_table_revision_data.with_raw_response.update_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
            item={"foo": "bar"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = response.parse()
        assert_matches_type(LookupTableRevisionDataUpdateKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    def test_streaming_response_update_key(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.with_streaming_response.update_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
            item={"foo": "bar"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = response.parse()
            assert_matches_type(LookupTableRevisionDataUpdateKeyResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_key(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.update_key(
                lookup_key="lookupKey",
                lookup_table_id="",
                lookup_table_revision_id="lookupTableRevisionId",
                item={"foo": "bar"},
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.update_key(
                lookup_key="lookupKey",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="",
                item={"foo": "bar"},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_key` but received ''"):
            client.lookup_tables.lookup_table_revision_data.with_raw_response.update_key(
                lookup_key="",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
                item={"foo": "bar"},
            )


class TestAsyncLookupTableRevisionData:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.retrieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )
        assert_matches_type(LookupTableRevisionDataRetrieveResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.retrieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            additional=["string"],
            limit=0,
        )
        assert_matches_type(LookupTableRevisionDataRetrieveResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = await response.parse()
        assert_matches_type(LookupTableRevisionDataRetrieveResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.with_streaming_response.retrieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = await response.parse()
            assert_matches_type(LookupTableRevisionDataRetrieveResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.update(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            items=[{"foo": "bar"}],
        )
        assert_matches_type(LookupTableRevisionDataUpdateResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.update(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            items=[{"foo": "bar"}],
            additional=["string"],
            version=0,
        )
        assert_matches_type(LookupTableRevisionDataUpdateResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.update(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            items=[{"foo": "bar"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = await response.parse()
        assert_matches_type(LookupTableRevisionDataUpdateResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.with_streaming_response.update(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            items=[{"foo": "bar"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = await response.parse()
            assert_matches_type(LookupTableRevisionDataUpdateResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.update(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
                items=[{"foo": "bar"}],
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.update(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
                items=[{"foo": "bar"}],
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.delete(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )
        assert_matches_type(LookupTableRevisionDataDeleteResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.delete(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = await response.parse()
        assert_matches_type(LookupTableRevisionDataDeleteResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.with_streaming_response.delete(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = await response.parse()
            assert_matches_type(LookupTableRevisionDataDeleteResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.delete(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.delete(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
            )

    @parametrize
    async def test_method_archieve(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.archieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_type="application/jsonl",
        )
        assert_matches_type(LookupTableRevisionDataArchieveResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_raw_response_archieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.archieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_type="application/jsonl",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = await response.parse()
        assert_matches_type(LookupTableRevisionDataArchieveResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_streaming_response_archieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.with_streaming_response.archieve(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_type="application/jsonl",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = await response.parse()
            assert_matches_type(LookupTableRevisionDataArchieveResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_archieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.archieve(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
                content_type="application/jsonl",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.archieve(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
                content_type="application/jsonl",
            )

    @parametrize
    async def test_method_copy(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.copy(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )
        assert_matches_type(LookupTableRevisionDataCopyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_method_copy_with_all_params(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.copy(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            revision_id="revisionId",
        )
        assert_matches_type(LookupTableRevisionDataCopyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_raw_response_copy(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.copy(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = await response.parse()
        assert_matches_type(LookupTableRevisionDataCopyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_streaming_response_copy(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.with_streaming_response.copy(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = await response.parse()
            assert_matches_type(LookupTableRevisionDataCopyResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_copy(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.copy(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.copy(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
            )

    @parametrize
    async def test_method_delete_key(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.delete_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        )
        assert_matches_type(LookupTableRevisionDataDeleteKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_method_delete_key_with_all_params(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.delete_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
            version=0,
        )
        assert_matches_type(LookupTableRevisionDataDeleteKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_raw_response_delete_key(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.delete_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = await response.parse()
        assert_matches_type(LookupTableRevisionDataDeleteKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_streaming_response_delete_key(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.with_streaming_response.delete_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = await response.parse()
            assert_matches_type(LookupTableRevisionDataDeleteKeyResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete_key(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.delete_key(
                lookup_key="lookupKey",
                lookup_table_id="",
                lookup_table_revision_id="lookupTableRevisionId",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.delete_key(
                lookup_key="lookupKey",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_key` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.delete_key(
                lookup_key="",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )

    @parametrize
    async def test_method_generate_download_url(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.generate_download_url(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_length=1,
            content_type="application/jsonl",
            file_name="x",
        )
        assert_matches_type(
            LookupTableRevisionDataGenerateDownloadURLResponse, lookup_table_revision_data, path=["response"]
        )

    @parametrize
    async def test_method_generate_download_url_with_all_params(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.generate_download_url(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_length=1,
            content_type="application/jsonl",
            file_name="x",
            version=0,
        )
        assert_matches_type(
            LookupTableRevisionDataGenerateDownloadURLResponse, lookup_table_revision_data, path=["response"]
        )

    @parametrize
    async def test_raw_response_generate_download_url(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.generate_download_url(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_length=1,
            content_type="application/jsonl",
            file_name="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = await response.parse()
        assert_matches_type(
            LookupTableRevisionDataGenerateDownloadURLResponse, lookup_table_revision_data, path=["response"]
        )

    @parametrize
    async def test_streaming_response_generate_download_url(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.with_streaming_response.generate_download_url(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_length=1,
            content_type="application/jsonl",
            file_name="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = await response.parse()
            assert_matches_type(
                LookupTableRevisionDataGenerateDownloadURLResponse, lookup_table_revision_data, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_generate_download_url(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.generate_download_url(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
                content_length=1,
                content_type="application/jsonl",
                file_name="x",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.generate_download_url(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
                content_length=1,
                content_type="application/jsonl",
                file_name="x",
            )

    @parametrize
    async def test_method_retrieve_key(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.retrieve_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        )
        assert_matches_type(LookupTableRevisionDataRetrieveKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_raw_response_retrieve_key(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = await response.parse()
        assert_matches_type(LookupTableRevisionDataRetrieveKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve_key(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.with_streaming_response.retrieve_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = await response.parse()
            assert_matches_type(
                LookupTableRevisionDataRetrieveKeyResponse, lookup_table_revision_data, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve_key(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve_key(
                lookup_key="lookupKey",
                lookup_table_id="",
                lookup_table_revision_id="lookupTableRevisionId",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve_key(
                lookup_key="lookupKey",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_key` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.retrieve_key(
                lookup_key="",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )

    @parametrize
    async def test_method_update_key(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.update_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
            item={"foo": "bar"},
        )
        assert_matches_type(LookupTableRevisionDataUpdateKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_method_update_key_with_all_params(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data = await async_client.lookup_tables.lookup_table_revision_data.update_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
            item={"foo": "bar"},
            additional=["string"],
            version=0,
        )
        assert_matches_type(LookupTableRevisionDataUpdateKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_raw_response_update_key(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.update_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
            item={"foo": "bar"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data = await response.parse()
        assert_matches_type(LookupTableRevisionDataUpdateKeyResponse, lookup_table_revision_data, path=["response"])

    @parametrize
    async def test_streaming_response_update_key(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.with_streaming_response.update_key(
            lookup_key="lookupKey",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
            item={"foo": "bar"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data = await response.parse()
            assert_matches_type(LookupTableRevisionDataUpdateKeyResponse, lookup_table_revision_data, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_key(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.update_key(
                lookup_key="lookupKey",
                lookup_table_id="",
                lookup_table_revision_id="lookupTableRevisionId",
                item={"foo": "bar"},
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.update_key(
                lookup_key="lookupKey",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="",
                item={"foo": "bar"},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_key` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.with_raw_response.update_key(
                lookup_key="",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
                item={"foo": "bar"},
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.balances import (
    transaction_schedule_list_params,
    transaction_schedule_create_params,
    transaction_schedule_update_params,
    transaction_schedule_preview_params,
)
from ...types.balances.schedule_response import ScheduleResponse

__all__ = ["TransactionSchedulesResource", "AsyncTransactionSchedulesResource"]


class TransactionSchedulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TransactionSchedulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return TransactionSchedulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TransactionSchedulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return TransactionSchedulesResourceWithStreamingResponse(self)

    def create(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        amount: float,
        code: str,
        end_date: Union[str, datetime],
        frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        frequency_interval: int,
        name: str,
        start_date: Union[str, datetime],
        transaction_description: str,
        transaction_type_id: str,
        currency_paid: str | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        paid: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduleResponse:
        """
        Create a new BalanceTransactionSchedule.

        Args:
          amount: The amount of each Balance Transaction created by this Schedule.

          code: The unique short code of the Balance Transaction Schedule.

          end_date: The end date (_in ISO-8601 format_) of the Balance Transaction Schedule.

              **NOTE:** End date is exclusive.

          frequency: Represents standard scheduling frequencies options for a job.

          frequency_interval: Used in conjunction with `frequency` to define how often Balance Transactions
              are generated by the Schedule. For example, if `frequency` is MONTHLY and
              `frequencyInterval` is 3, then Balance Transactions are generated every three
              months.

          name: The name of the Balance Transaction Schedule.

          start_date: The start date (_in ISO-8601 format_) of the Balance Transaction Schedule.

          transaction_description: The description of each Balance Transaction that will be created by this
              Schedule.

          transaction_type_id: The unique identifier (UUID) of the transaction type used to create Transactions
              by this Schedule. You can obtain a list of Transaction Types created for the
              Organization. See the the
              [List TransactionTypes](https://www.m3ter.com/docs/api#tag/TransactionType/operation/ListTransactionTypes)
              endpoint of this API Reference.

          currency_paid: The currency code of the payment if it differs from the Balance currency. For
              example: USD, GBP or EUR.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          paid: The payment amount if the payment currency differs from the Balance currency.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._post(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules",
            body=maybe_transform(
                {
                    "amount": amount,
                    "code": code,
                    "end_date": end_date,
                    "frequency": frequency,
                    "frequency_interval": frequency_interval,
                    "name": name,
                    "start_date": start_date,
                    "transaction_description": transaction_description,
                    "transaction_type_id": transaction_type_id,
                    "currency_paid": currency_paid,
                    "custom_fields": custom_fields,
                    "paid": paid,
                    "version": version,
                },
                transaction_schedule_create_params.TransactionScheduleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduleResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduleResponse:
        """
        Retrieve a BalanceTransactionSchedule for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduleResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        amount: float,
        code: str,
        end_date: Union[str, datetime],
        frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        frequency_interval: int,
        name: str,
        start_date: Union[str, datetime],
        transaction_description: str,
        transaction_type_id: str,
        currency_paid: str | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        paid: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduleResponse:
        """
        Update a BalanceTransactionSchedule for the given UUID.

        Args:
          amount: The amount of each Balance Transaction created by this Schedule.

          code: The unique short code of the Balance Transaction Schedule.

          end_date: The end date (_in ISO-8601 format_) of the Balance Transaction Schedule.

              **NOTE:** End date is exclusive.

          frequency: Represents standard scheduling frequencies options for a job.

          frequency_interval: Used in conjunction with `frequency` to define how often Balance Transactions
              are generated by the Schedule. For example, if `frequency` is MONTHLY and
              `frequencyInterval` is 3, then Balance Transactions are generated every three
              months.

          name: The name of the Balance Transaction Schedule.

          start_date: The start date (_in ISO-8601 format_) of the Balance Transaction Schedule.

          transaction_description: The description of each Balance Transaction that will be created by this
              Schedule.

          transaction_type_id: The unique identifier (UUID) of the transaction type used to create Transactions
              by this Schedule. You can obtain a list of Transaction Types created for the
              Organization. See the the
              [List TransactionTypes](https://www.m3ter.com/docs/api#tag/TransactionType/operation/ListTransactionTypes)
              endpoint of this API Reference.

          currency_paid: The currency code of the payment if it differs from the Balance currency. For
              example: USD, GBP or EUR.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          paid: The payment amount if the payment currency differs from the Balance currency.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules/{id}",
            body=maybe_transform(
                {
                    "amount": amount,
                    "code": code,
                    "end_date": end_date,
                    "frequency": frequency,
                    "frequency_interval": frequency_interval,
                    "name": name,
                    "start_date": start_date,
                    "transaction_description": transaction_description,
                    "transaction_type_id": transaction_type_id,
                    "currency_paid": currency_paid,
                    "custom_fields": custom_fields,
                    "paid": paid,
                    "version": version,
                },
                transaction_schedule_update_params.TransactionScheduleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduleResponse,
        )

    def list(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[ScheduleResponse]:
        """
        Retrieve a list of BalanceTransactionSchedule entities.

        Args:
          next_token: nextToken for multi page retrievals

          page_size: Number of BalanceTransactionSchedules to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules",
            page=SyncCursor[ScheduleResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    transaction_schedule_list_params.TransactionScheduleListParams,
                ),
            ),
            model=ScheduleResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduleResponse:
        """
        Delete the BalanceTransactionSchedule for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduleResponse,
        )

    def preview(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        amount: float,
        code: str,
        end_date: Union[str, datetime],
        frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        frequency_interval: int,
        name: str,
        start_date: Union[str, datetime],
        transaction_description: str,
        transaction_type_id: str,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        currency_paid: str | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        paid: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduleResponse:
        """
        Previews the BalanceTransactions this Schedule would create, without persisting
        them. You can use this call to obtain a preview of the Transactions a Schedule
        you plan to create for a Balance would generate.

        Args:
          amount: The amount of each Balance Transaction created by this Schedule.

          code: The unique short code of the Balance Transaction Schedule.

          end_date: The end date (_in ISO-8601 format_) of the Balance Transaction Schedule.

              **NOTE:** End date is exclusive.

          frequency: Represents standard scheduling frequencies options for a job.

          frequency_interval: Used in conjunction with `frequency` to define how often Balance Transactions
              are generated by the Schedule. For example, if `frequency` is MONTHLY and
              `frequencyInterval` is 3, then Balance Transactions are generated every three
              months.

          name: The name of the Balance Transaction Schedule.

          start_date: The start date (_in ISO-8601 format_) of the Balance Transaction Schedule.

          transaction_description: The description of each Balance Transaction that will be created by this
              Schedule.

          transaction_type_id: The unique identifier (UUID) of the transaction type used to create Transactions
              by this Schedule. You can obtain a list of Transaction Types created for the
              Organization. See the the
              [List TransactionTypes](https://www.m3ter.com/docs/api#tag/TransactionType/operation/ListTransactionTypes)
              endpoint of this API Reference.

          next_token: nextToken for multi page retrievals

          page_size: Number of BalanceTransactions to retrieve per page

          currency_paid: The currency code of the payment if it differs from the Balance currency. For
              example: USD, GBP or EUR.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          paid: The payment amount if the payment currency differs from the Balance currency.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._post(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules/preview",
            body=maybe_transform(
                {
                    "amount": amount,
                    "code": code,
                    "end_date": end_date,
                    "frequency": frequency,
                    "frequency_interval": frequency_interval,
                    "name": name,
                    "start_date": start_date,
                    "transaction_description": transaction_description,
                    "transaction_type_id": transaction_type_id,
                    "currency_paid": currency_paid,
                    "custom_fields": custom_fields,
                    "paid": paid,
                    "version": version,
                },
                transaction_schedule_preview_params.TransactionSchedulePreviewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    transaction_schedule_preview_params.TransactionSchedulePreviewParams,
                ),
            ),
            cast_to=ScheduleResponse,
        )


class AsyncTransactionSchedulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTransactionSchedulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTransactionSchedulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTransactionSchedulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncTransactionSchedulesResourceWithStreamingResponse(self)

    async def create(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        amount: float,
        code: str,
        end_date: Union[str, datetime],
        frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        frequency_interval: int,
        name: str,
        start_date: Union[str, datetime],
        transaction_description: str,
        transaction_type_id: str,
        currency_paid: str | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        paid: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduleResponse:
        """
        Create a new BalanceTransactionSchedule.

        Args:
          amount: The amount of each Balance Transaction created by this Schedule.

          code: The unique short code of the Balance Transaction Schedule.

          end_date: The end date (_in ISO-8601 format_) of the Balance Transaction Schedule.

              **NOTE:** End date is exclusive.

          frequency: Represents standard scheduling frequencies options for a job.

          frequency_interval: Used in conjunction with `frequency` to define how often Balance Transactions
              are generated by the Schedule. For example, if `frequency` is MONTHLY and
              `frequencyInterval` is 3, then Balance Transactions are generated every three
              months.

          name: The name of the Balance Transaction Schedule.

          start_date: The start date (_in ISO-8601 format_) of the Balance Transaction Schedule.

          transaction_description: The description of each Balance Transaction that will be created by this
              Schedule.

          transaction_type_id: The unique identifier (UUID) of the transaction type used to create Transactions
              by this Schedule. You can obtain a list of Transaction Types created for the
              Organization. See the the
              [List TransactionTypes](https://www.m3ter.com/docs/api#tag/TransactionType/operation/ListTransactionTypes)
              endpoint of this API Reference.

          currency_paid: The currency code of the payment if it differs from the Balance currency. For
              example: USD, GBP or EUR.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          paid: The payment amount if the payment currency differs from the Balance currency.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return await self._post(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules",
            body=await async_maybe_transform(
                {
                    "amount": amount,
                    "code": code,
                    "end_date": end_date,
                    "frequency": frequency,
                    "frequency_interval": frequency_interval,
                    "name": name,
                    "start_date": start_date,
                    "transaction_description": transaction_description,
                    "transaction_type_id": transaction_type_id,
                    "currency_paid": currency_paid,
                    "custom_fields": custom_fields,
                    "paid": paid,
                    "version": version,
                },
                transaction_schedule_create_params.TransactionScheduleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduleResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduleResponse:
        """
        Retrieve a BalanceTransactionSchedule for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduleResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        amount: float,
        code: str,
        end_date: Union[str, datetime],
        frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        frequency_interval: int,
        name: str,
        start_date: Union[str, datetime],
        transaction_description: str,
        transaction_type_id: str,
        currency_paid: str | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        paid: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduleResponse:
        """
        Update a BalanceTransactionSchedule for the given UUID.

        Args:
          amount: The amount of each Balance Transaction created by this Schedule.

          code: The unique short code of the Balance Transaction Schedule.

          end_date: The end date (_in ISO-8601 format_) of the Balance Transaction Schedule.

              **NOTE:** End date is exclusive.

          frequency: Represents standard scheduling frequencies options for a job.

          frequency_interval: Used in conjunction with `frequency` to define how often Balance Transactions
              are generated by the Schedule. For example, if `frequency` is MONTHLY and
              `frequencyInterval` is 3, then Balance Transactions are generated every three
              months.

          name: The name of the Balance Transaction Schedule.

          start_date: The start date (_in ISO-8601 format_) of the Balance Transaction Schedule.

          transaction_description: The description of each Balance Transaction that will be created by this
              Schedule.

          transaction_type_id: The unique identifier (UUID) of the transaction type used to create Transactions
              by this Schedule. You can obtain a list of Transaction Types created for the
              Organization. See the the
              [List TransactionTypes](https://www.m3ter.com/docs/api#tag/TransactionType/operation/ListTransactionTypes)
              endpoint of this API Reference.

          currency_paid: The currency code of the payment if it differs from the Balance currency. For
              example: USD, GBP or EUR.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          paid: The payment amount if the payment currency differs from the Balance currency.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules/{id}",
            body=await async_maybe_transform(
                {
                    "amount": amount,
                    "code": code,
                    "end_date": end_date,
                    "frequency": frequency,
                    "frequency_interval": frequency_interval,
                    "name": name,
                    "start_date": start_date,
                    "transaction_description": transaction_description,
                    "transaction_type_id": transaction_type_id,
                    "currency_paid": currency_paid,
                    "custom_fields": custom_fields,
                    "paid": paid,
                    "version": version,
                },
                transaction_schedule_update_params.TransactionScheduleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduleResponse,
        )

    def list(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ScheduleResponse, AsyncCursor[ScheduleResponse]]:
        """
        Retrieve a list of BalanceTransactionSchedule entities.

        Args:
          next_token: nextToken for multi page retrievals

          page_size: Number of BalanceTransactionSchedules to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules",
            page=AsyncCursor[ScheduleResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    transaction_schedule_list_params.TransactionScheduleListParams,
                ),
            ),
            model=ScheduleResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduleResponse:
        """
        Delete the BalanceTransactionSchedule for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduleResponse,
        )

    async def preview(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        amount: float,
        code: str,
        end_date: Union[str, datetime],
        frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        frequency_interval: int,
        name: str,
        start_date: Union[str, datetime],
        transaction_description: str,
        transaction_type_id: str,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        currency_paid: str | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        paid: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduleResponse:
        """
        Previews the BalanceTransactions this Schedule would create, without persisting
        them. You can use this call to obtain a preview of the Transactions a Schedule
        you plan to create for a Balance would generate.

        Args:
          amount: The amount of each Balance Transaction created by this Schedule.

          code: The unique short code of the Balance Transaction Schedule.

          end_date: The end date (_in ISO-8601 format_) of the Balance Transaction Schedule.

              **NOTE:** End date is exclusive.

          frequency: Represents standard scheduling frequencies options for a job.

          frequency_interval: Used in conjunction with `frequency` to define how often Balance Transactions
              are generated by the Schedule. For example, if `frequency` is MONTHLY and
              `frequencyInterval` is 3, then Balance Transactions are generated every three
              months.

          name: The name of the Balance Transaction Schedule.

          start_date: The start date (_in ISO-8601 format_) of the Balance Transaction Schedule.

          transaction_description: The description of each Balance Transaction that will be created by this
              Schedule.

          transaction_type_id: The unique identifier (UUID) of the transaction type used to create Transactions
              by this Schedule. You can obtain a list of Transaction Types created for the
              Organization. See the the
              [List TransactionTypes](https://www.m3ter.com/docs/api#tag/TransactionType/operation/ListTransactionTypes)
              endpoint of this API Reference.

          next_token: nextToken for multi page retrievals

          page_size: Number of BalanceTransactions to retrieve per page

          currency_paid: The currency code of the payment if it differs from the Balance currency. For
              example: USD, GBP or EUR.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          paid: The payment amount if the payment currency differs from the Balance currency.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return await self._post(
            f"/organizations/{org_id}/balances/{balance_id}/balancetransactionschedules/preview",
            body=await async_maybe_transform(
                {
                    "amount": amount,
                    "code": code,
                    "end_date": end_date,
                    "frequency": frequency,
                    "frequency_interval": frequency_interval,
                    "name": name,
                    "start_date": start_date,
                    "transaction_description": transaction_description,
                    "transaction_type_id": transaction_type_id,
                    "currency_paid": currency_paid,
                    "custom_fields": custom_fields,
                    "paid": paid,
                    "version": version,
                },
                transaction_schedule_preview_params.TransactionSchedulePreviewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    transaction_schedule_preview_params.TransactionSchedulePreviewParams,
                ),
            ),
            cast_to=ScheduleResponse,
        )


class TransactionSchedulesResourceWithRawResponse:
    def __init__(self, transaction_schedules: TransactionSchedulesResource) -> None:
        self._transaction_schedules = transaction_schedules

        self.create = to_raw_response_wrapper(
            transaction_schedules.create,
        )
        self.retrieve = to_raw_response_wrapper(
            transaction_schedules.retrieve,
        )
        self.update = to_raw_response_wrapper(
            transaction_schedules.update,
        )
        self.list = to_raw_response_wrapper(
            transaction_schedules.list,
        )
        self.delete = to_raw_response_wrapper(
            transaction_schedules.delete,
        )
        self.preview = to_raw_response_wrapper(
            transaction_schedules.preview,
        )


class AsyncTransactionSchedulesResourceWithRawResponse:
    def __init__(self, transaction_schedules: AsyncTransactionSchedulesResource) -> None:
        self._transaction_schedules = transaction_schedules

        self.create = async_to_raw_response_wrapper(
            transaction_schedules.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            transaction_schedules.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            transaction_schedules.update,
        )
        self.list = async_to_raw_response_wrapper(
            transaction_schedules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            transaction_schedules.delete,
        )
        self.preview = async_to_raw_response_wrapper(
            transaction_schedules.preview,
        )


class TransactionSchedulesResourceWithStreamingResponse:
    def __init__(self, transaction_schedules: TransactionSchedulesResource) -> None:
        self._transaction_schedules = transaction_schedules

        self.create = to_streamed_response_wrapper(
            transaction_schedules.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            transaction_schedules.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            transaction_schedules.update,
        )
        self.list = to_streamed_response_wrapper(
            transaction_schedules.list,
        )
        self.delete = to_streamed_response_wrapper(
            transaction_schedules.delete,
        )
        self.preview = to_streamed_response_wrapper(
            transaction_schedules.preview,
        )


class AsyncTransactionSchedulesResourceWithStreamingResponse:
    def __init__(self, transaction_schedules: AsyncTransactionSchedulesResource) -> None:
        self._transaction_schedules = transaction_schedules

        self.create = async_to_streamed_response_wrapper(
            transaction_schedules.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            transaction_schedules.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            transaction_schedules.update,
        )
        self.list = async_to_streamed_response_wrapper(
            transaction_schedules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            transaction_schedules.delete,
        )
        self.preview = async_to_streamed_response_wrapper(
            transaction_schedules.preview,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["LookupTableRevisionDataArchieveResponse"]


class LookupTableRevisionDataArchieveResponse(BaseModel):
    """Response containing the archive job URL details"""

    expiry: Optional[datetime] = None
    """The URL expiry time"""

    url: Optional[str] = None
    """The presigned URL"""
